@@ -1,5 +1,4 @@
 """Apache Cassandra result store backend using the DataStax driver."""
-import sys
 import threading
 
 from celery import states
@@ -14,7 +13,7 @@ try:  # pragma: no cover
     import cassandra.cluster
     import cassandra.query
 except ImportError:  # pragma: no cover
-    cassandra = None   # noqa
+    cassandra = None
 
 
 __all__ = ('CassandraBackend',)
@@ -60,11 +59,9 @@ Q_EXPIRES = """
     USING TTL {0}
 """
 
-if sys.version_info[0] == 3:
-    def buf_t(x):
-        return bytes(x, 'utf8')
-else:
-    buf_t = buffer  # noqa
+
+def buf_t(x):
+    return bytes(x, 'utf8')
 
 
 class CassandraBackend(BaseBackend):
