@@ -1,7 +1,8 @@
 """The ``celery result`` program, used to inspect task results."""
 import click
 
-from celery.bin.base import CeleryCommand, CeleryOption
+from celery.bin.base import (CeleryCommand, CeleryOption,
+                             handle_preload_options)
 
 
 @click.command(cls=CeleryCommand)
@@ -17,6 +18,7 @@ from celery.bin.base import CeleryCommand, CeleryOption
               help_group='Result Options',
               help="Show traceback instead.")
 @click.pass_context
+@handle_preload_options
 def result(ctx, task_id, task, traceback):
     """Print the return value for a given task id."""
     app = ctx.obj.app
