@@ -1,60 +1,60 @@
-# -*- coding: utf-8 -*-
+"""In-memory representation of cluster state.
+
+This module implements a data-structure used to keep
+track of the state of a cluster of workers and the tasks
+it is working on (by consuming events).
+
+For every event consumed the state is updated,
+so the state represents the state of the cluster
+at the time of the last event.
+
+Snapshots (:mod:`celery.events.snapshot`) can be used to
+take "pictures" of this state at regular intervals
+to for example, store that in a database.
 """
-    celery.events.state
-    ~~~~~~~~~~~~~~~~~~~
-
-    This module implements a datastructure used to keep
-    track of the state of a cluster of workers and the tasks
-    it is working on (by consuming events).
-
-    For every event consumed the state is updated,
-    so the state represents the state of the cluster
-    at the time of the last event.
-
-    Snapshots (:mod:`celery.events.snapshot`) can be used to
-    take "pictures" of this state at regular intervals
-    to e.g. store that in a database.
-
-"""
-from __future__ import absolute_import
-
 import bisect
 import sys
 import threading
-
+from collections import defaultdict
+from collections.abc import Callable
 from datetime import datetime
 from decimal import Decimal
 from itertools import islice
 from operator import itemgetter
 from time import time
-from weakref import ref
+from typing import Mapping
+from weakref import WeakSet, ref
 
 from kombu.clocks import timetuple
-from kombu.utils import cached_property, kwdict
+from kombu.utils.objects import cached_property
 
 from celery import states
-from celery.five import class_property, items, values
-from celery.utils import deprecated
-from celery.utils.functional import LRUCache
+from celery.utils.functional import LRUCache, memoize, pass1
 from celery.utils.log import get_logger
 
+__all__ = ('Worker', 'Task', 'State', 'heartbeat_expires')
+
+# pylint: disable=redefined-outer-name
+# We cache globals and attribute lookups, so disable this warning.
+# pylint: disable=too-many-function-args
+# For some reason pylint thinks ._event is a method, when it's a property.
+
+#: Set if running PyPy
 PYPY = hasattr(sys, 'pypy_version_info')
 
-# The window (in percentage) is added to the workers heartbeat
-# frequency.  If the time between updates exceeds this window,
-# then the worker is considered to be offline.
+#: The window (in percentage) is added to the workers heartbeat
+#: frequency.  If the time between updates exceeds this window,
+#: then the worker is considered to be offline.
 HEARTBEAT_EXPIRE_WINDOW = 200
 
-# Max drift between event timestamp and time of event received
-# before we alert that clocks may be unsynchronized.
+#: Max drift between event timestamp and time of event received
+#: before we alert that clocks may be unsynchronized.
 HEARTBEAT_DRIFT_MAX = 16
 
-DRIFT_WARNING = """\
-Substantial drift from %s may mean clocks are out of sync.  Current drift is
-%s seconds.  [orig: %s recv: %s]
-"""
-
-CAN_KWDICT = sys.version_info >= (2, 6, 5)
+DRIFT_WARNING = (
+    "Substantial drift from %s may mean clocks are out of sync.  Current drift is "
+    "%s seconds.  [orig: %s recv: %s]"
+)
 
 logger = get_logger(__name__)
 warn = logger.warning
@@ -63,14 +63,60 @@ R_STATE = '<State: events={0.event_count} tasks={0.task_count}>'
 R_WORKER = '<Worker: {0.hostname} ({0.status_string} clock:{0.clock})'
 R_TASK = '<Task: {0.name}({0.uuid}) {0.state} clock:{0.clock}>'
 
-__all__ = ['Worker', 'Task', 'State', 'heartbeat_expires']
+#: Mapping of task event names to task state.
+TASK_EVENT_TO_STATE = {
+    'sent': states.PENDING,
+    'received': states.RECEIVED,
+    'started': states.STARTED,
+    'failed': states.FAILURE,
+    'retried': states.RETRY,
+    'succeeded': states.SUCCESS,
+    'revoked': states.REVOKED,
+    'rejected': states.REJECTED,
+}
+
+
+class CallableDefaultdict(defaultdict):
+    """:class:`~collections.defaultdict` with configurable __call__.
+
+    We use this for backwards compatibility in State.tasks_by_type
+    etc, which used to be a method but is now an index instead.
+
+    So you can do::
+
+        >>> add_tasks = state.tasks_by_type['proj.tasks.add']
+
+    while still supporting the method call::
+
+        >>> add_tasks = list(state.tasks_by_type(
+        ...     'proj.tasks.add', reverse=True))
+    """
+
+    def __init__(self, fun, *args, **kwargs):
+        self.fun = fun
+        super().__init__(*args, **kwargs)
+
+    def __call__(self, *args, **kwargs):
+        return self.fun(*args, **kwargs)
+
+
+Callable.register(CallableDefaultdict)
+
+
+@memoize(maxsize=1000, keyfun=lambda a, _: a[0])
+def _warn_drift(hostname, drift, local_received, timestamp):
+    # we use memoize here so the warning is only logged once per hostname
+    warn(DRIFT_WARNING, hostname, drift,
+         datetime.fromtimestamp(local_received),
+         datetime.fromtimestamp(timestamp))
 
 
 def heartbeat_expires(timestamp, freq=60,
                       expire_window=HEARTBEAT_EXPIRE_WINDOW,
                       Decimal=Decimal, float=float, isinstance=isinstance):
+    """Return time when heartbeat expires."""
     # some json implementations returns decimal.Decimal objects,
-    # which are not compatible with float.
+    # which aren't compatible with float.
     freq = float(freq) if isinstance(freq, Decimal) else freq
     if isinstance(timestamp, Decimal):
         timestamp = float(timestamp)
@@ -78,7 +124,7 @@ def heartbeat_expires(timestamp, freq=60,
 
 
 def _depickle_task(cls, fields):
-    return cls(**(fields if CAN_KWDICT else kwdict(fields)))
+    return cls(**fields)
 
 
 def with_unique_field(attr):
@@ -92,7 +138,8 @@ def with_unique_field(attr):
         cls.__eq__ = __eq__
 
         def __ne__(this, other):
-            return not this.__eq__(other)
+            res = this.__eq__(other)
+            return True if res is NotImplemented else not res
         cls.__ne__ = __ne__
 
         def __hash__(this):
@@ -104,15 +151,16 @@ def with_unique_field(attr):
 
 
 @with_unique_field('hostname')
-class Worker(object):
+class Worker:
     """Worker State."""
+
     heartbeat_max = 4
     expire_window = HEARTBEAT_EXPIRE_WINDOW
 
     _fields = ('hostname', 'pid', 'freq', 'heartbeats', 'clock',
                'active', 'processed', 'loadavg', 'sw_ident',
                'sw_ver', 'sw_sys')
-    if not PYPY:
+    if not PYPY:  # pragma: no cover
         __slots__ = _fields + ('event', '__dict__', '__weakref__')
 
     def __init__(self, hostname=None, pid=None, freq=60,
@@ -146,10 +194,10 @@ class Worker(object):
 
         def event(type_, timestamp=None,
                   local_received=None, fields=None,
-                  max_drift=HEARTBEAT_DRIFT_MAX, items=items, abs=abs, int=int,
+                  max_drift=HEARTBEAT_DRIFT_MAX, abs=abs, int=int,
                   insort=bisect.insort, len=len):
             fields = fields or {}
-            for k, v in items(fields):
+            for k, v in fields.items():
                 _set(self, k, v)
             if type_ == 'offline':
                 heartbeats[:] = []
@@ -157,11 +205,10 @@ class Worker(object):
                 if not local_received or not timestamp:
                     return
                 drift = abs(int(local_received) - int(timestamp))
-                if drift > HEARTBEAT_DRIFT_MAX:
-                    warn(DRIFT_WARNING, self.hostname, drift,
-                         datetime.fromtimestamp(local_received),
-                         datetime.fromtimestamp(timestamp))
-                if local_received:
+                if drift > max_drift:
+                    _warn_drift(self.hostname, drift,
+                                local_received, timestamp)
+                if local_received:  # pragma: no cover
                     hearts = len(heartbeats)
                     if hearts > hbmax - 1:
                         hb_pop(0)
@@ -172,7 +219,8 @@ class Worker(object):
         return event
 
     def update(self, f, **kw):
-        for k, v in items(dict(f, **kw) if kw else f):
+        d = dict(f, **kw) if kw else f
+        for k, v in d.items():
             setattr(self, k, v)
 
     def __repr__(self):
@@ -195,91 +243,82 @@ class Worker(object):
     def id(self):
         return '{0.hostname}.{0.pid}'.format(self)
 
-    @deprecated(3.2, 3.3)
-    def update_heartbeat(self, received, timestamp):
-        self.event(None, timestamp, received)
-
-    @deprecated(3.2, 3.3)
-    def on_online(self, timestamp=None, local_received=None, **fields):
-        self.event('online', timestamp, local_received, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_offline(self, timestamp=None, local_received=None, **fields):
-        self.event('offline', timestamp, local_received, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_heartbeat(self, timestamp=None, local_received=None, **fields):
-        self.event('heartbeat', timestamp, local_received, fields)
-
-    @class_property
-    def _defaults(cls):
-        """Deprecated, to be removed in 3.3"""
-        source = cls()
-        return dict((k, getattr(source, k)) for k in cls._fields)
-
 
 @with_unique_field('uuid')
-class Task(object):
+class Task:
     """Task State."""
+
     name = received = sent = started = succeeded = failed = retried = \
-        revoked = args = kwargs = eta = expires = retries = worker = result = \
-        exception = timestamp = runtime = traceback = exchange = \
-        routing_key = client = None
+        revoked = rejected = args = kwargs = eta = expires = retries = \
+        worker = result = exception = timestamp = runtime = traceback = \
+        exchange = routing_key = root_id = parent_id = client = None
     state = states.PENDING
     clock = 0
 
-    _fields = ('uuid', 'name', 'state', 'received', 'sent', 'started',
-               'succeeded', 'failed', 'retried', 'revoked', 'args', 'kwargs',
-               'eta', 'expires', 'retries', 'worker', 'result', 'exception',
-               'timestamp', 'runtime', 'traceback', 'exchange', 'routing_key',
-               'clock', 'client')
-    if not PYPY:
+    _fields = (
+        'uuid', 'name', 'state', 'received', 'sent', 'started', 'rejected',
+        'succeeded', 'failed', 'retried', 'revoked', 'args', 'kwargs',
+        'eta', 'expires', 'retries', 'worker', 'result', 'exception',
+        'timestamp', 'runtime', 'traceback', 'exchange', 'routing_key',
+        'clock', 'client', 'root', 'root_id', 'parent', 'parent_id',
+        'children',
+    )
+    if not PYPY:  # pragma: no cover
         __slots__ = ('__dict__', '__weakref__')
 
     #: How to merge out of order events.
-    #: Disorder is detected by logical ordering (e.g. :event:`task-received`
-    #: must have happened before a :event:`task-failed` event).
+    #: Disorder is detected by logical ordering (e.g., :event:`task-received`
+    #: must've happened before a :event:`task-failed` event).
     #:
     #: A merge rule consists of a state and a list of fields to keep from
     #: that state. ``(RECEIVED, ('name', 'args')``, means the name and args
     #: fields are always taken from the RECEIVED state, and any values for
     #: these fields received before or after is simply ignored.
-    merge_rules = {states.RECEIVED: ('name', 'args', 'kwargs',
-                                     'retries', 'eta', 'expires')}
+    merge_rules = {
+        states.RECEIVED: (
+            'name', 'args', 'kwargs', 'parent_id',
+            'root_id', 'retries', 'eta', 'expires',
+        ),
+    }
 
     #: meth:`info` displays these fields by default.
-    _info_fields = ('args', 'kwargs', 'retries', 'result', 'eta', 'runtime',
-                    'expires', 'exception', 'exchange', 'routing_key')
+    _info_fields = (
+        'args', 'kwargs', 'retries', 'result', 'eta', 'runtime',
+        'expires', 'exception', 'exchange', 'routing_key',
+        'root_id', 'parent_id',
+    )
 
-    def __init__(self, uuid=None, **kwargs):
+    def __init__(self, uuid=None, cluster_state=None, children=None, **kwargs):
         self.uuid = uuid
+        self.cluster_state = cluster_state
+        if self.cluster_state is not None:
+            self.children = WeakSet(
+                self.cluster_state.tasks.get(task_id)
+                for task_id in children or ()
+                if task_id in self.cluster_state.tasks
+            )
+        else:
+            self.children = WeakSet()
+        self._serializer_handlers = {
+            'children': self._serializable_children,
+            'root': self._serializable_root,
+            'parent': self._serializable_parent,
+        }
         if kwargs:
-            for k, v in items(kwargs):
-                setattr(self, k, v)
+            self.__dict__.update(kwargs)
 
     def event(self, type_, timestamp=None, local_received=None, fields=None,
-              precedence=states.precedence, items=items, dict=dict,
-              PENDING=states.PENDING, RECEIVED=states.RECEIVED,
-              STARTED=states.STARTED, FAILURE=states.FAILURE,
-              RETRY=states.RETRY, SUCCESS=states.SUCCESS,
-              REVOKED=states.REVOKED):
+              precedence=states.precedence, setattr=setattr,
+              task_event_to_state=TASK_EVENT_TO_STATE.get, RETRY=states.RETRY):
         fields = fields or {}
-        if type_ == 'sent':
-            state, self.sent = PENDING, timestamp
-        elif type_ == 'received':
-            state, self.received = RECEIVED, timestamp
-        elif type_ == 'started':
-            state, self.started = STARTED, timestamp
-        elif type_ == 'failed':
-            state, self.failed = FAILURE, timestamp
-        elif type_ == 'retried':
-            state, self.retried = RETRY, timestamp
-        elif type_ == 'succeeded':
-            state, self.succeeded = SUCCESS, timestamp
-        elif type_ == 'revoked':
-            state, self.revoked = REVOKED, timestamp
+
+        # using .get is faster than catching KeyError in this case.
+        state = task_event_to_state(type_)
+        if state is not None:
+            # sets, for example, self.succeeded to the timestamp.
+            setattr(self, type_, timestamp)
         else:
-            state = type_.upper()
+            state = type_.upper()  # custom state
 
         # note that precedence here is reversed
         # see implementation in celery.states.state.__lt__
@@ -288,19 +327,18 @@ class Task(object):
             # this state logically happens-before the current state, so merge.
             keep = self.merge_rules.get(state)
             if keep is not None:
-                fields = dict(
-                    (k, v) for k, v in items(fields) if k in keep
-                )
-            for key, value in items(fields):
-                setattr(self, key, value)
+                fields = {
+                    k: v for k, v in fields.items() if k in keep
+                }
         else:
-            self.state = state
-            self.timestamp = timestamp
-            for key, value in items(fields):
-                setattr(self, key, value)
+            fields.update(state=state, timestamp=timestamp)
 
-    def info(self, fields=None, extra=[]):
+        # update current state with info from this event.
+        self.__dict__.update(fields)
+
+    def info(self, fields=None, extra=None):
         """Information about this task suitable for on-screen display."""
+        extra = [] if not extra else extra
         fields = self._info_fields if fields is None else fields
 
         def _keys():
@@ -316,13 +354,27 @@ class Task(object):
 
     def as_dict(self):
         get = object.__getattribute__
-        return dict(
-            (k, get(self, k)) for k in self._fields
-        )
+        handler = self._serializer_handlers.get
+        return {
+            k: handler(k, pass1)(get(self, k)) for k in self._fields
+        }
+
+    def _serializable_children(self, value):
+        return [task.id for task in self.children]
+
+    def _serializable_root(self, value):
+        return self.root_id
+
+    def _serializable_parent(self, value):
+        return self.parent_id
 
     def __reduce__(self):
         return _depickle_task, (self.__class__, self.as_dict())
 
+    @property
+    def id(self):
+        return self.uuid
+
     @property
     def origin(self):
         return self.client if self.worker is None else self.worker.id
@@ -331,60 +383,26 @@ class Task(object):
     def ready(self):
         return self.state in states.READY_STATES
 
-    @deprecated(3.2, 3.3)
-    def on_sent(self, timestamp=None, **fields):
-        self.event('sent', timestamp, fields)
+    @cached_property
+    def parent(self):
+        # issue github.com/mher/flower/issues/648
+        try:
+            return self.parent_id and self.cluster_state.tasks.data[self.parent_id]
+        except KeyError:
+            return None
 
-    @deprecated(3.2, 3.3)
-    def on_received(self, timestamp=None, **fields):
-        self.event('received', timestamp, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_started(self, timestamp=None, **fields):
-        self.event('started', timestamp, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_failed(self, timestamp=None, **fields):
-        self.event('failed', timestamp, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_retried(self, timestamp=None, **fields):
-        self.event('retried', timestamp, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_succeeded(self, timestamp=None, **fields):
-        self.event('succeeded', timestamp, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_revoked(self, timestamp=None, **fields):
-        self.event('revoked', timestamp, fields)
-
-    @deprecated(3.2, 3.3)
-    def on_unknown_event(self, shortype, timestamp=None, **fields):
-        self.event(shortype, timestamp, fields)
-
-    @deprecated(3.2, 3.3)
-    def update(self, state, timestamp, fields,
-               _state=states.state, RETRY=states.RETRY):
-        return self.event(state, timestamp, None, fields)
-
-    @deprecated(3.2, 3.3)
-    def merge(self, state, timestamp, fields):
-        keep = self.merge_rules.get(state)
-        if keep is not None:
-            fields = dict((k, v) for k, v in items(fields) if k in keep)
-        for key, value in items(fields):
-            setattr(self, key, value)
-
-    @class_property
-    def _defaults(cls):
-        """Deprecated, to be removed in 3.3."""
-        source = cls()
-        return dict((k, getattr(source, k)) for k in source._fields)
+    @cached_property
+    def root(self):
+        # issue github.com/mher/flower/issues/648
+        try:
+            return self.root_id and self.cluster_state.tasks.data[self.root_id]
+        except KeyError:
+            return None
 
 
-class State(object):
+class State:
     """Records clusters state."""
+
     Worker = Worker
     Task = Task
     event_count = 0
@@ -394,7 +412,8 @@ class State(object):
     def __init__(self, callback=None,
                  workers=None, tasks=None, taskheap=None,
                  max_workers_in_memory=5000, max_tasks_in_memory=10000,
-                 on_node_join=None, on_node_leave=None):
+                 on_node_join=None, on_node_leave=None,
+                 tasks_by_type=None, tasks_by_worker=None):
         self.event_callback = callback
         self.workers = (LRUCache(max_workers_in_memory)
                         if workers is None else workers)
@@ -408,8 +427,19 @@ class State(object):
         self._mutex = threading.Lock()
         self.handlers = {}
         self._seen_types = set()
+        self._tasks_to_resolve = {}
         self.rebuild_taskheap()
 
+        self.tasks_by_type = CallableDefaultdict(
+            self._tasks_by_type, WeakSet)  # type: Mapping[str, WeakSet[Task]]
+        self.tasks_by_type.update(
+            _deserialize_Task_WeakSet_Mapping(tasks_by_type, self.tasks))
+
+        self.tasks_by_worker = CallableDefaultdict(
+            self._tasks_by_worker, WeakSet)  # type: Mapping[str, WeakSet[Task]]
+        self.tasks_by_worker.update(
+            _deserialize_Task_WeakSet_Mapping(tasks_by_worker, self.tasks))
+
     @cached_property
     def _event(self):
         return self._create_dispatcher()
@@ -429,9 +459,10 @@ class State(object):
 
     def _clear_tasks(self, ready=True):
         if ready:
-            in_progress = dict(
-                (uuid, task) for uuid, task in self.itertasks()
-                if task.state not in states.READY_STATES)
+            in_progress = {
+                uuid: task for uuid, task in self.itertasks()
+                if task.state not in states.READY_STATES
+            }
             self.tasks.clear()
             self.tasks.update(in_progress)
         else:
@@ -451,7 +482,8 @@ class State(object):
     def get_or_create_worker(self, hostname, **kwargs):
         """Get or create worker by hostname.
 
-        Return tuple of ``(worker, was_created)``.
+        Returns:
+            Tuple: of ``(worker, was_created)`` pairs.
         """
         try:
             worker = self.workers[hostname]
@@ -468,7 +500,7 @@ class State(object):
         try:
             return self.tasks[uuid], False
         except KeyError:
-            task = self.tasks[uuid] = self.Task(uuid)
+            task = self.tasks[uuid] = self.Task(uuid, cluster_state=self)
             return task, True
 
     def event(self, event):
@@ -484,6 +516,9 @@ class State(object):
         return self._event(dict(fields, type='-'.join(['worker', type_])))[0]
 
     def _create_dispatcher(self):
+
+        # pylint: disable=too-many-statements
+        # This code is highly optimized, but not for reusability.
         get_handler = self.handlers.__getitem__
         event_callback = self.event_callback
         wfields = itemgetter('hostname', 'timestamp', 'local_received')
@@ -504,6 +539,9 @@ class State(object):
         # avoid updating LRU entry at getitem
         get_worker, get_task = workers.data.__getitem__, tasks.data.__getitem__
 
+        get_task_by_type_set = self.tasks_by_type.__getitem__
+        get_task_by_worker_set = self.tasks_by_worker.__getitem__
+
         def _event(event,
                    timetuple=timetuple, KeyError=KeyError,
                    insort=bisect.insort, created=True):
@@ -545,14 +583,15 @@ class State(object):
                 # task-sent event is sent by client, not worker
                 is_client_event = subject == 'sent'
                 try:
-                    task, created = get_task(uuid), False
+                    task, task_created = get_task(uuid), False
                 except KeyError:
-                    task = tasks[uuid] = Task(uuid)
+                    task = tasks[uuid] = Task(uuid, cluster_state=self)
+                    task_created = True
                 if is_client_event:
                     task.client = hostname
                 else:
                     try:
-                        worker, created = get_worker(hostname), False
+                        worker = get_worker(hostname)
                     except KeyError:
                         worker = workers[hostname] = Worker(hostname)
                     task.worker = worker
@@ -579,27 +618,58 @@ class State(object):
                 task_name = task.name
                 if task_name is not None:
                     add_type(task_name)
-                return (task, created), subject
+                    if task_created:  # add to tasks_by_type index
+                        get_task_by_type_set(task_name).add(task)
+                        get_task_by_worker_set(hostname).add(task)
+                if task.parent_id:
+                    try:
+                        parent_task = self.tasks[task.parent_id]
+                    except KeyError:
+                        self._add_pending_task_child(task)
+                    else:
+                        parent_task.children.add(task)
+                try:
+                    _children = self._tasks_to_resolve.pop(uuid)
+                except KeyError:
+                    pass
+                else:
+                    task.children.update(_children)
+
+                return (task, task_created), subject
         return _event
 
+    def _add_pending_task_child(self, task):
+        try:
+            ch = self._tasks_to_resolve[task.parent_id]
+        except KeyError:
+            ch = self._tasks_to_resolve[task.parent_id] = WeakSet()
+        ch.add(task)
+
     def rebuild_taskheap(self, timetuple=timetuple):
         heap = self._taskheap[:] = [
             timetuple(t.clock, t.timestamp, t.origin, ref(t))
-            for t in values(self.tasks)
+            for t in self.tasks.values()
         ]
         heap.sort()
 
     def itertasks(self, limit=None):
-        for index, row in enumerate(items(self.tasks)):
+        for index, row in enumerate(self.tasks.items()):
             yield row
             if limit and index + 1 >= limit:
                 break
 
-    def tasks_by_time(self, limit=None):
-        """Generator giving tasks ordered by time,
-        in ``(uuid, Task)`` tuples."""
+    def tasks_by_time(self, limit=None, reverse=True):
+        """Generator yielding tasks ordered by time.
+
+        Yields:
+            Tuples of ``(uuid, Task)``.
+        """
+        _heap = self._taskheap
+        if reverse:
+            _heap = reversed(_heap)
+
         seen = set()
-        for evtup in islice(reversed(self._taskheap), 0, limit):
+        for evtup in islice(_heap, 0, limit):
             task = evtup[3]()
             if task is not None:
                 uuid = task.uuid
@@ -608,24 +678,28 @@ class State(object):
                     seen.add(uuid)
     tasks_by_timestamp = tasks_by_time
 
-    def tasks_by_type(self, name, limit=None):
+    def _tasks_by_type(self, name, limit=None, reverse=True):
         """Get all tasks by type.
 
-        Return a list of ``(uuid, Task)`` tuples.
+        This is slower than accessing :attr:`tasks_by_type`,
+        but will be ordered by time.
 
+        Returns:
+            Generator: giving ``(uuid, Task)`` pairs.
         """
         return islice(
-            ((uuid, task) for uuid, task in self.tasks_by_time()
+            ((uuid, task) for uuid, task in self.tasks_by_time(reverse=reverse)
              if task.name == name),
             0, limit,
         )
 
-    def tasks_by_worker(self, hostname, limit=None):
+    def _tasks_by_worker(self, hostname, limit=None, reverse=True):
         """Get all tasks by worker.
 
+        Slower than accessing :attr:`tasks_by_worker`, but ordered by time.
         """
         return islice(
-            ((uuid, task) for uuid, task in self.tasks_by_time()
+            ((uuid, task) for uuid, task in self.tasks_by_time(reverse=reverse)
              if task.worker.hostname == hostname),
             0, limit,
         )
@@ -636,7 +710,7 @@ class State(object):
 
     def alive_workers(self):
         """Return a list of (seemingly) alive workers."""
-        return [w for w in values(self.workers) if w.alive]
+        return (w for w in self.workers.values() if w.alive)
 
     def __repr__(self):
         return R_STATE.format(self)
@@ -646,4 +720,16 @@ class State(object):
             self.event_callback, self.workers, self.tasks, None,
             self.max_workers_in_memory, self.max_tasks_in_memory,
             self.on_node_join, self.on_node_leave,
+            _serialize_Task_WeakSet_Mapping(self.tasks_by_type),
+            _serialize_Task_WeakSet_Mapping(self.tasks_by_worker),
         )
+
+
+def _serialize_Task_WeakSet_Mapping(mapping):
+    return {name: [t.id for t in tasks] for name, tasks in mapping.items()}
+
+
+def _deserialize_Task_WeakSet_Mapping(mapping, tasks):
+    mapping = mapping or {}
+    return {name: WeakSet(tasks[i] for i in ids if i in tasks)
+            for name, ids in mapping.items()}
