@@ -1,36 +1,24 @@
-# -*- coding: utf-8 -*-
-"""
-    celery.loaders.base
-    ~~~~~~~~~~~~~~~~~~~
-
-    Loader base class.
-
-"""
-from __future__ import absolute_import
-
-import anyjson
-import imp as _imp
+"""Loader base class."""
 import importlib
 import os
 import re
 import sys
-
 from datetime import datetime
 
-from kombu.utils import cached_property
-from kombu.utils.encoding import safe_str
+from kombu.utils import json
+from kombu.utils.objects import cached_property
 
 from celery import signals
-from celery.datastructures import DictAttribute, force_mapping
-from celery.five import reraise, string_t
+from celery.exceptions import reraise
+from celery.utils.collections import DictAttribute, force_mapping
 from celery.utils.functional import maybe_list
-from celery.utils.imports import (
-    import_from_cwd, symbol_by_name, NotAPackage, find_module,
-)
+from celery.utils.imports import (NotAPackage, find_module, import_from_cwd,
+                                  symbol_by_name)
 
-__all__ = ['BaseLoader']
+__all__ = ('BaseLoader',)
 
 _RACE_PROTECTION = False
+
 CONFIG_INVALID_NAME = """\
 Error: Module '{module}' doesn't exist, or it's not a valid \
 Python module name.
@@ -40,9 +28,11 @@ CONFIG_WITH_SUFFIX = CONFIG_INVALID_NAME + """\
 Did you mean '{suggest}'?
 """
 
+unconfigured = object()
 
-class BaseLoader(object):
-    """The base class for loaders.
+
+class BaseLoader:
+    """Base class for loaders.
 
     Loaders handles,
 
@@ -58,14 +48,14 @@ class BaseLoader(object):
             See :meth:`on_worker_shutdown`.
 
         * What modules are imported to find tasks?
-
     """
+
     builtin_modules = frozenset()
     configured = False
     override_backends = {}
     worker_initialized = False
 
-    _conf = None
+    _conf = unconfigured
 
     def __init__(self, app, **kwargs):
         self.app = app
@@ -77,26 +67,19 @@ class BaseLoader(object):
         return datetime.now()
 
     def on_task_init(self, task_id, task):
-        """This method is called before a task is executed."""
-        pass
+        """Called before a task is executed."""
 
     def on_process_cleanup(self):
-        """This method is called after a task is executed."""
-        pass
+        """Called after a task is executed."""
 
     def on_worker_init(self):
-        """This method is called when the worker (:program:`celery worker`)
-        starts."""
-        pass
+        """Called when the worker (:program:`celery worker`) starts."""
 
     def on_worker_shutdown(self):
-        """This method is called when the worker (:program:`celery worker`)
-        shuts down."""
-        pass
+        """Called when the worker (:program:`celery worker`) shuts down."""
 
     def on_worker_process_init(self):
-        """This method is called when a child process starts."""
-        pass
+        """Called when a child process starts."""
 
     def import_task_module(self, module):
         self.task_modules.add(module)
@@ -113,14 +96,14 @@ class BaseLoader(object):
         )
 
     def import_default_modules(self):
-        signals.import_modules.send(sender=self.app)
-        return [
-            self.import_task_module(m) for m in (
-                tuple(self.builtin_modules) +
-                tuple(maybe_list(self.app.conf.CELERY_IMPORTS)) +
-                tuple(maybe_list(self.app.conf.CELERY_INCLUDE))
-            )
-        ]
+        responses = signals.import_modules.send(sender=self.app)
+        # Prior to this point loggers are not yet set up properly, need to
+        #   check responses manually and reraised exceptions if any, otherwise
+        #   they'll be silenced, making it incredibly difficult to debug.
+        for _, response in responses:
+            if isinstance(response, Exception):
+                raise response
+        return [self.import_task_module(m) for m in self.default_modules]
 
     def init_worker(self):
         if not self.worker_initialized:
@@ -135,7 +118,7 @@ class BaseLoader(object):
         self.on_worker_process_init()
 
     def config_from_object(self, obj, silent=False):
-        if isinstance(obj, string_t):
+        if isinstance(obj, str):
             try:
                 obj = self._smart_import(obj, imp=self.import_from_cwd)
             except (ImportError, AttributeError):
@@ -143,17 +126,19 @@ class BaseLoader(object):
                     return False
                 raise
         self._conf = force_mapping(obj)
+        if self._conf.get('override_backends') is not None:
+            self.override_backends = self._conf['override_backends']
         return True
 
     def _smart_import(self, path, imp=None):
         imp = self.import_module if imp is None else imp
         if ':' in path:
-            # Path includes attribute so can just jump here.
-            # e.g. ``os.path:abspath``.
+            # Path includes attribute so can just jump
+            # here (e.g., ``os.path:abspath``).
             return symbol_by_name(path, imp=imp)
 
         # Not sure if path is just a module name or if it includes an
-        # attribute name (e.g. ``os.path``, vs, ``os.path.abspath``
+        # attribute name (e.g., ``os.path``, vs, ``os.path.abspath``).
         try:
             return imp(path)
         except ImportError:
@@ -163,44 +148,44 @@ class BaseLoader(object):
     def _import_config_module(self, name):
         try:
             self.find_module(name)
-        except NotAPackage:
+        except NotAPackage as exc:
             if name.endswith('.py'):
                 reraise(NotAPackage, NotAPackage(CONFIG_WITH_SUFFIX.format(
-                    module=name, suggest=name[:-3])), sys.exc_info()[2])
-            reraise(NotAPackage, NotAPackage(CONFIG_INVALID_NAME.format(
-                module=name)), sys.exc_info()[2])
+                        module=name, suggest=name[:-3])), sys.exc_info()[2])
+            raise NotAPackage(CONFIG_INVALID_NAME.format(module=name)) from exc
         else:
             return self.import_from_cwd(name)
 
     def find_module(self, module):
         return find_module(module)
 
-    def cmdline_config_parser(
-            self, args, namespace='celery',
-            re_type=re.compile(r'\((\w+)\)'),
-            extra_types={'json': anyjson.loads},
-            override_types={'tuple': 'json',
-                            'list': 'json',
-                            'dict': 'json'}):
-        from celery.app.defaults import Option, NAMESPACES
-        namespace = namespace.upper()
+    def cmdline_config_parser(self, args, namespace='celery',
+                              re_type=re.compile(r'\((\w+)\)'),
+                              extra_types=None,
+                              override_types=None):
+        extra_types = extra_types if extra_types else {'json': json.loads}
+        override_types = override_types if override_types else {
+            'tuple': 'json',
+            'list': 'json',
+            'dict': 'json'
+        }
+        from celery.app.defaults import NAMESPACES, Option
+        namespace = namespace and namespace.lower()
         typemap = dict(Option.typemap, **extra_types)
 
         def getarg(arg):
-            """Parse a single configuration definition from
-            the command-line."""
-
+            """Parse single configuration from command-line."""
             # ## find key/value
             # ns.key=value|ns_key=value (case insensitive)
             key, value = arg.split('=', 1)
-            key = key.upper().replace('.', '_')
+            key = key.lower().replace('.', '_')
 
-            # ## find namespace.
-            # .key=value|_key=value expands to default namespace.
+            # ## find name-space.
+            # .key=value|_key=value expands to default name-space.
             if key[0] == '_':
                 ns, key = namespace, key[1:]
             else:
-                # find namespace part of key
+                # find name-space part of key
                 ns, key = key.split('_', 1)
 
             ns_key = (ns and ns + '_' or '') + key
@@ -214,26 +199,13 @@ class BaseLoader(object):
                 value = typemap[type_](value)
             else:
                 try:
-                    value = NAMESPACES[ns][key].to_python(value)
+                    value = NAMESPACES[ns.lower()][key].to_python(value)
                 except ValueError as exc:
                     # display key name in error message.
-                    raise ValueError('{0!r}: {1}'.format(ns_key, exc))
+                    raise ValueError(f'{ns_key!r}: {exc}')
             return ns_key, value
         return dict(getarg(arg) for arg in args)
 
-    def mail_admins(self, subject, body, fail_silently=False,
-                    sender=None, to=None, host=None, port=None,
-                    user=None, password=None, timeout=None,
-                    use_ssl=False, use_tls=False):
-        message = self.mail.Message(sender=sender, to=to,
-                                    subject=safe_str(subject),
-                                    body=safe_str(body))
-        mailer = self.mail.Mailer(host=host, port=port,
-                                  user=user, password=password,
-                                  timeout=timeout, use_ssl=use_ssl,
-                                  use_tls=use_tls)
-        mailer.send(message, fail_silently=fail_silently)
-
     def read_configuration(self, env='CELERY_CONFIG_MODULE'):
         try:
             custom_config = os.environ[env]
@@ -243,24 +215,27 @@ class BaseLoader(object):
             if custom_config:
                 usercfg = self._import_config_module(custom_config)
                 return DictAttribute(usercfg)
-        return {}
 
     def autodiscover_tasks(self, packages, related_name='tasks'):
         self.task_modules.update(
             mod.__name__ for mod in autodiscover_tasks(packages or (),
                                                        related_name) if mod)
 
+    @cached_property
+    def default_modules(self):
+        return (
+            tuple(self.builtin_modules) +
+            tuple(maybe_list(self.app.conf.imports)) +
+            tuple(maybe_list(self.app.conf.include))
+        )
+
     @property
     def conf(self):
         """Loader configuration."""
-        if self._conf is None:
+        if self._conf is unconfigured:
             self._conf = self.read_configuration()
         return self._conf
 
-    @cached_property
-    def mail(self):
-        return self.import_module('celery.utils.mail')
-
 
 def autodiscover_tasks(packages, related_name='tasks'):
     global _RACE_PROTECTION
@@ -275,24 +250,24 @@ def autodiscover_tasks(packages, related_name='tasks'):
 
 
 def find_related_module(package, related_name):
-    """Given a package name and a module name, tries to find that
-    module."""
-
-    # Django 1.7 allows for speciying a class name in INSTALLED_APPS.
+    """Find module in package."""
+    # Django 1.7 allows for specifying a class name in INSTALLED_APPS.
     # (Issue #2248).
     try:
-        importlib.import_module(package)
+        module = importlib.import_module(package)
+        if not related_name and module:
+            return module
     except ImportError:
         package, _, _ = package.rpartition('.')
+        if not package:
+            raise
+
+    module_name = f'{package}.{related_name}'
 
     try:
-        pkg_path = importlib.import_module(package).__path__
-    except AttributeError:
+        return importlib.import_module(module_name)
+    except ImportError as e:
+        import_exc_name = getattr(e, 'name', module_name)
+        if import_exc_name is not None and import_exc_name != module_name:
+            raise e
         return
-
-    try:
-        _imp.find_module(related_name, pkg_path)
-    except ImportError:
-        return
-
-    return importlib.import_module('{0}.{1}'.format(package, related_name))
