@@ -1,4 +1,4 @@
-from django.conf.urls import handler404, handler500, include, url  # noqa
+from django.urls import handler404, handler500, include, url  # noqa
 
 # Uncomment the next two lines to enable the admin:
 # from django.contrib import admin
