@@ -1,14 +1,9 @@
-from __future__ import absolute_import, unicode_literals
-
 import sys
 from importlib import import_module
 
-from case import mock
-
 from celery.app.defaults import (_OLD_DEFAULTS, _OLD_SETTING_KEYS,
                                  _TO_NEW_KEY, _TO_OLD_KEY, DEFAULTS,
                                  NAMESPACES, SETTING_KEYS)
-from celery.five import values
 
 
 class test_defaults:
@@ -27,16 +22,6 @@ class test_defaults:
         val = object()
         assert self.defaults.Option.typemap['any'](val) is val
 
-    @mock.sys_platform('darwin')
-    @mock.pypy_version((1, 4, 0))
-    def test_default_pool_pypy_14(self):
-        assert self.defaults.DEFAULT_POOL == 'solo'
-
-    @mock.sys_platform('darwin')
-    @mock.pypy_version((1, 5, 0))
-    def test_default_pool_pypy_15(self):
-        assert self.defaults.DEFAULT_POOL == 'prefork'
-
     def test_compat_indices(self):
         assert not any(key.isupper() for key in DEFAULTS)
         assert not any(key.islower() for key in _OLD_DEFAULTS)
@@ -44,8 +29,8 @@ class test_defaults:
         assert not any(key.islower() for key in _TO_NEW_KEY)
         assert not any(key.isupper() for key in SETTING_KEYS)
         assert not any(key.islower() for key in _OLD_SETTING_KEYS)
-        assert not any(value.isupper() for value in values(_TO_NEW_KEY))
-        assert not any(value.islower() for value in values(_TO_OLD_KEY))
+        assert not any(value.isupper() for value in _TO_NEW_KEY.values())
+        assert not any(value.islower() for value in _TO_OLD_KEY.values())
 
         for key in _TO_NEW_KEY:
             assert key in _OLD_SETTING_KEYS
