@@ -6,8 +6,6 @@ from tempfile import mktemp
 from unittest.mock import Mock, patch
 
 import pytest
-from case import mock
-from case.utils import get_logger_handlers
 
 from celery import signals, uuid
 from celery.app.log import TaskFormatter
@@ -15,6 +13,7 @@ from celery.utils.log import (ColorFormatter, LoggingProxy, get_logger,
                               get_task_logger, in_sighandler)
 from celery.utils.log import logger as base_logger
 from celery.utils.log import logger_isa, task_logger
+from t.unit import conftest
 
 
 class test_TaskFormatter:
@@ -165,12 +164,10 @@ class test_default_logger:
         logger = get_logger(base_logger.name)
         assert logger.parent is logging.root
 
-    @mock.restore_logging()
-    def test_setup_logging_subsystem_misc(self):
+    def test_setup_logging_subsystem_misc(self, restore_logging):
         self.app.log.setup_logging_subsystem(loglevel=None)
 
-    @mock.restore_logging()
-    def test_setup_logging_subsystem_misc2(self):
+    def test_setup_logging_subsystem_misc2(self, restore_logging):
         self.app.conf.worker_hijack_root_logger = True
         self.app.log.setup_logging_subsystem()
 
@@ -183,18 +180,15 @@ class test_default_logger:
         self.app.log._configure_logger(None, sys.stderr, None, '', False)
         logger.handlers[:] = []
 
-    @mock.restore_logging()
-    def test_setup_logging_subsystem_colorize(self):
+    def test_setup_logging_subsystem_colorize(self, restore_logging):
         self.app.log.setup_logging_subsystem(colorize=None)
         self.app.log.setup_logging_subsystem(colorize=True)
 
-    @mock.restore_logging()
-    def test_setup_logging_subsystem_no_mputil(self):
-        with mock.mask_modules('billiard.util'):
-            self.app.log.setup_logging_subsystem()
+    @pytest.mark.masked_modules('billiard.util')
+    def test_setup_logging_subsystem_no_mputil(self, restore_logging, mask_modules):
+        self.app.log.setup_logging_subsystem()
 
-    @mock.restore_logging()
-    def test_setup_logger(self):
+    def test_setup_logger(self, restore_logging):
         logger = self.setup_logger(loglevel=logging.ERROR, logfile=None,
                                    root=False, colorize=True)
         logger.handlers = []
@@ -202,16 +196,14 @@ class test_default_logger:
         logger = self.setup_logger(loglevel=logging.ERROR, logfile=None,
                                    root=False, colorize=None)
         # setup_logger logs to stderr without logfile argument.
-        assert (get_logger_handlers(logger)[0].stream is
+        assert (conftest.get_logger_handlers(logger)[0].stream is
                 sys.__stderr__)
 
-    @mock.restore_logging()
-    def test_setup_logger_no_handlers_stream(self):
+    def test_setup_logger_no_handlers_stream(self, restore_logging):
         l = self.get_logger()
         l.handlers = []
 
-        with mock.stdouts() as outs:
-            stdout, stderr = outs
+        with conftest.stdouts() as (stdout, stderr):
             l = self.setup_logger(logfile=sys.stderr,
                                   loglevel=logging.INFO, root=False)
             l.info('The quick brown fox...')
@@ -221,7 +213,7 @@ class test_default_logger:
     def test_setup_logger_no_handlers_file(self, *args):
         tempfile = mktemp(suffix='unittest', prefix='celery')
         with patch('builtins.open') as osopen:
-            with mock.restore_logging():
+            with conftest.restore_logging_context_manager():
                 files = defaultdict(StringIO)
 
                 def open_file(filename, *args, **kwargs):
@@ -236,16 +228,15 @@ class test_default_logger:
                 l = self.setup_logger(
                     logfile=tempfile, loglevel=logging.INFO, root=False,
                 )
-                assert isinstance(get_logger_handlers(l)[0],
+                assert isinstance(conftest.get_logger_handlers(l)[0],
                                   logging.FileHandler)
                 assert tempfile in files
 
-    @mock.restore_logging()
-    def test_redirect_stdouts(self):
+    def test_redirect_stdouts(self, restore_logging):
         logger = self.setup_logger(loglevel=logging.ERROR, logfile=None,
                                    root=False)
         try:
-            with mock.wrap_logger(logger) as sio:
+            with conftest.wrap_logger(logger) as sio:
                 self.app.log.redirect_stdouts_to_logger(
                     logger, loglevel=logging.ERROR,
                 )
@@ -257,19 +248,20 @@ class test_default_logger:
         finally:
             sys.stdout, sys.stderr = sys.__stdout__, sys.__stderr__
 
-    @mock.restore_logging()
-    def test_logging_proxy(self):
+    def test_logging_proxy(self, restore_logging):
         logger = self.setup_logger(loglevel=logging.ERROR, logfile=None,
                                    root=False)
 
-        with mock.wrap_logger(logger) as sio:
+        with conftest.wrap_logger(logger) as sio:
             p = LoggingProxy(logger, loglevel=logging.ERROR)
             p.close()
             p.write('foo')
             assert 'foo' not in sio.getvalue()
             p.closed = False
+            p.write('\n')
+            assert sio.getvalue() == ''
             write_res = p.write('foo ')
-            assert 'foo ' in sio.getvalue()
+            assert sio.getvalue() == 'foo \n'
             assert write_res == 4
             lines = ['baz', 'xuzzy']
             p.writelines(lines)
@@ -279,13 +271,36 @@ class test_default_logger:
             p.close()
             assert not p.isatty()
 
-            with mock.stdouts() as (stdout, stderr):
+            with conftest.stdouts() as (stdout, stderr):
                 with in_sighandler():
                     p.write('foo')
                     assert stderr.getvalue()
 
-    @mock.restore_logging()
-    def test_logging_proxy_recurse_protection(self):
+    def test_logging_proxy_bytes(self, restore_logging):
+        logger = self.setup_logger(loglevel=logging.ERROR, logfile=None,
+                                   root=False)
+
+        with conftest.wrap_logger(logger) as sio:
+            p = LoggingProxy(logger, loglevel=logging.ERROR)
+            p.close()
+            p.write(b'foo')
+            assert 'foo' not in str(sio.getvalue())
+            p.closed = False
+            p.write(b'\n')
+            assert str(sio.getvalue()) == ''
+            write_res = p.write(b'foo ')
+            assert str(sio.getvalue()) == 'foo \n'
+            assert write_res == 4
+            p.flush()
+            p.close()
+            assert not p.isatty()
+
+            with conftest.stdouts() as (stdout, stderr):
+                with in_sighandler():
+                    p.write(b'foo')
+                    assert stderr.getvalue()
+
+    def test_logging_proxy_recurse_protection(self, restore_logging):
         logger = self.setup_logger(loglevel=logging.ERROR, logfile=None,
                                    root=False)
         p = LoggingProxy(logger, loglevel=logging.ERROR)
@@ -336,7 +351,7 @@ class MockLogger(logging.Logger):
 
     def __init__(self, *args, **kwargs):
         self._records = []
-        logging.Logger.__init__(self, *args, **kwargs)
+        super().__init__(*args, **kwargs)
 
     def handle(self, record):
         self._records.append(record)
