@@ -2,6 +2,8 @@ import datetime
 from pickle import dumps, loads
 from unittest.mock import ANY, MagicMock, Mock, patch, sentinel
 
+import dns.version
+import pymongo
 import pytest
 import pytz
 from kombu.exceptions import EncodeError
@@ -25,10 +27,44 @@ MONGODB_PASSWORD = '1234'
 MONGODB_DATABASE = 'testing'
 MONGODB_COLLECTION = 'collection1'
 MONGODB_GROUP_COLLECTION = 'group_collection1'
+# uri with user, password, database name, replica set, DNS seedlist format
+MONGODB_SEEDLIST_URI = ('srv://'
+                        'celeryuser:celerypassword@'
+                        'dns-seedlist-host.example.com/'
+                        'celerydatabase')
+MONGODB_BACKEND_HOST = [
+    'mongo1.example.com:27017',
+    'mongo2.example.com:27017',
+    'mongo3.example.com:27017',
+]
+CELERY_USER = 'celeryuser'
+CELERY_PASSWORD = 'celerypassword'
+CELERY_DATABASE = 'celerydatabase'
 
 pytest.importorskip('pymongo')
 
 
+def fake_resolver_dnspython():
+    TXT = pytest.importorskip('dns.rdtypes.ANY.TXT').TXT
+    SRV = pytest.importorskip('dns.rdtypes.IN.SRV').SRV
+
+    def mock_resolver(_, rdtype, rdclass=None, lifetime=None, **kwargs):
+
+        if rdtype == 'SRV':
+            return [
+                SRV(0, 0, 0, 0, 27017, hostname)
+                for hostname in [
+                    'mongo1.example.com',
+                    'mongo2.example.com',
+                    'mongo3.example.com'
+                ]
+            ]
+        elif rdtype == 'TXT':
+            return [TXT(0, 0, [b'replicaSet=rs0'])]
+
+    return mock_resolver
+
+
 class test_MongoBackend:
     default_url = 'mongodb://uuuu:pwpw@hostname.dom/database'
     replica_set_url = (
@@ -86,18 +122,14 @@ class test_MongoBackend:
                'mongo3.example.com:27017/'
                'celerydatabase?replicaSet=rs0')
         mb = MongoBackend(app=self.app, url=uri)
-        assert mb.mongo_host == [
-            'mongo1.example.com:27017',
-            'mongo2.example.com:27017',
-            'mongo3.example.com:27017',
-        ]
+        assert mb.mongo_host == MONGODB_BACKEND_HOST
         assert mb.options == dict(
             mb._prepare_client_options(),
             replicaset='rs0',
         )
-        assert mb.user == 'celeryuser'
-        assert mb.password == 'celerypassword'
-        assert mb.database_name == 'celerydatabase'
+        assert mb.user == CELERY_USER
+        assert mb.password == CELERY_PASSWORD
+        assert mb.database_name == CELERY_DATABASE
 
         # same uri, change some parameters in backend settings
         self.app.conf.mongodb_backend_settings = {
@@ -109,65 +141,93 @@ class test_MongoBackend:
             },
         }
         mb = MongoBackend(app=self.app, url=uri)
-        assert mb.mongo_host == [
-            'mongo1.example.com:27017',
-            'mongo2.example.com:27017',
-            'mongo3.example.com:27017',
-        ]
+        assert mb.mongo_host == MONGODB_BACKEND_HOST
         assert mb.options == dict(
             mb._prepare_client_options(),
             replicaset='rs1',
             socketKeepAlive=True,
         )
         assert mb.user == 'backenduser'
-        assert mb.password == 'celerypassword'
+        assert mb.password == CELERY_PASSWORD
         assert mb.database_name == 'another_db'
 
         mb = MongoBackend(app=self.app, url='mongodb://')
 
-    def test_init_mongodb_dns_seedlist(self):
-        Name = pytest.importorskip('dns.name').Name
-        TXT = pytest.importorskip('dns.rdtypes.ANY.TXT').TXT
-        SRV = pytest.importorskip('dns.rdtypes.IN.SRV').SRV
-
+    @pytest.mark.skipif(dns.version.MAJOR > 1,
+                        reason="For dnspython version > 1, pymongo's"
+                               "srv_resolver calls resolver.resolve")
+    @pytest.mark.skipif(pymongo.version_tuple[0] > 3,
+                        reason="For pymongo version > 3, options returns ssl")
+    def test_init_mongodb_dnspython1_pymongo3_seedlist(self):
+        resolver = fake_resolver_dnspython()
         self.app.conf.mongodb_backend_settings = None
 
-        def mock_resolver(_, rdtype, rdclass=None, lifetime=None, **kwargs):
-
-            if rdtype == 'SRV':
-                return [
-                    SRV(0, 0, 0, 0, 27017, Name(labels=hostname))
-                    for hostname in [
-                        b'mongo1.example.com'.split(b'.'),
-                        b'mongo2.example.com'.split(b'.'),
-                        b'mongo3.example.com'.split(b'.')
-                    ]
-                ]
-            elif rdtype == 'TXT':
-                return [TXT(0, 0, [b'replicaSet=rs0'])]
-
-        # uri with user, password, database name, replica set,
-        # DNS seedlist format
-        uri = ('srv://'
-               'celeryuser:celerypassword@'
-               'dns-seedlist-host.example.com/'
-               'celerydatabase')
-
-        with patch('dns.resolver.query', side_effect=mock_resolver):
-            mb = MongoBackend(app=self.app, url=uri)
-            assert mb.mongo_host == [
-                'mongo1.example.com:27017',
-                'mongo2.example.com:27017',
-                'mongo3.example.com:27017',
-            ]
+        with patch('dns.resolver.query', side_effect=resolver):
+            mb = self.perform_seedlist_assertions()
             assert mb.options == dict(
                 mb._prepare_client_options(),
                 replicaset='rs0',
                 ssl=True
             )
-            assert mb.user == 'celeryuser'
-            assert mb.password == 'celerypassword'
-            assert mb.database_name == 'celerydatabase'
+
+    @pytest.mark.skipif(dns.version.MAJOR <= 1,
+                        reason="For dnspython versions 1.X, pymongo's"
+                               "srv_resolver calls resolver.query")
+    @pytest.mark.skipif(pymongo.version_tuple[0] > 3,
+                        reason="For pymongo version > 3, options returns ssl")
+    def test_init_mongodb_dnspython2_pymongo3_seedlist(self):
+        resolver = fake_resolver_dnspython()
+        self.app.conf.mongodb_backend_settings = None
+
+        with patch('dns.resolver.resolve', side_effect=resolver):
+            mb = self.perform_seedlist_assertions()
+            assert mb.options == dict(
+                mb._prepare_client_options(),
+                replicaset='rs0',
+                ssl=True
+            )
+
+    @pytest.mark.skipif(dns.version.MAJOR > 1,
+                        reason="For dnspython version >= 2, pymongo's"
+                               "srv_resolver calls resolver.resolve")
+    @pytest.mark.skipif(pymongo.version_tuple[0] <= 3,
+                        reason="For pymongo version > 3, options returns tls")
+    def test_init_mongodb_dnspython1_pymongo4_seedlist(self):
+        resolver = fake_resolver_dnspython()
+        self.app.conf.mongodb_backend_settings = None
+
+        with patch('dns.resolver.query', side_effect=resolver):
+            mb = self.perform_seedlist_assertions()
+            assert mb.options == dict(
+                mb._prepare_client_options(),
+                replicaset='rs0',
+                tls=True
+            )
+
+    @pytest.mark.skipif(dns.version.MAJOR <= 1,
+                        reason="For dnspython versions 1.X, pymongo's"
+                               "srv_resolver calls resolver.query")
+    @pytest.mark.skipif(pymongo.version_tuple[0] <= 3,
+                        reason="For pymongo version > 3, options returns tls")
+    def test_init_mongodb_dnspython2_pymongo4_seedlist(self):
+        resolver = fake_resolver_dnspython()
+        self.app.conf.mongodb_backend_settings = None
+
+        with patch('dns.resolver.resolve', side_effect=resolver):
+            mb = self.perform_seedlist_assertions()
+            assert mb.options == dict(
+                mb._prepare_client_options(),
+                replicaset='rs0',
+                tls=True
+            )
+
+    def perform_seedlist_assertions(self):
+        mb = MongoBackend(app=self.app, url=MONGODB_SEEDLIST_URI)
+        assert mb.mongo_host == MONGODB_BACKEND_HOST
+        assert mb.user == CELERY_USER
+        assert mb.password == CELERY_PASSWORD
+        assert mb.database_name == CELERY_DATABASE
+        return mb
 
     def test_ensure_mongodb_uri_compliance(self):
         mb = MongoBackend(app=self.app, url=None)
@@ -234,8 +294,8 @@ class test_MongoBackend:
             connection = mb._get_connection()
             mock_Connection.assert_called_once_with(
                 host=['localhost:27017'],
-                username='celeryuser',
-                password='celerypassword',
+                username=CELERY_USER,
+                password=CELERY_PASSWORD,
                 authmechanism='SCRAM-SHA-256',
                 **mb._prepare_client_options()
             )
@@ -273,8 +333,6 @@ class test_MongoBackend:
 
         assert database is mock_database
         assert self.backend.__dict__['database'] is mock_database
-        mock_database.authenticate.assert_called_once_with(
-            MONGODB_USER, MONGODB_PASSWORD, source=self.backend.database_name)
 
     @patch('celery.backends.mongodb.MongoBackend._get_connection')
     def test_get_database_no_existing_no_auth(self, mock_get_connection):
@@ -290,7 +348,6 @@ class test_MongoBackend:
         database = self.backend.database
 
         assert database is mock_database
-        mock_database.authenticate.assert_not_called()
         assert self.backend.__dict__['database'] is mock_database
 
     @patch('celery.backends.mongodb.MongoBackend._get_database')
@@ -489,19 +546,6 @@ class test_MongoBackend:
         self.backend.cleanup()
         mock_collection.delete_many.assert_not_called()
 
-    def test_get_database_authfailure(self):
-        x = MongoBackend(app=self.app)
-        x._get_connection = Mock()
-        conn = x._get_connection.return_value = {}
-        db = conn[x.database_name] = Mock()
-        db.authenticate.return_value = False
-        x.user = 'jerry'
-        x.password = 'cere4l'
-        with pytest.raises(ImproperlyConfigured):
-            x._get_database()
-        db.authenticate.assert_called_with('jerry', 'cere4l',
-                                           source=x.database_name)
-
     def test_prepare_client_options(self):
         with patch('pymongo.version_tuple', new=(3, 0, 3)):
             options = self.backend._prepare_client_options()
