@@ -1,13 +1,12 @@
-from __future__ import absolute_import, unicode_literals
+from unittest.mock import Mock, patch
 
 import pytest
-from case import Mock, mock, patch
 
 from celery.app.events import Events
 from celery.events.snapshot import Polaroid, evcam
 
 
-class MockTimer(object):
+class MockTimer:
     installed = []
 
     def call_repeatedly(self, secs, fun, *args, **kwargs):
@@ -90,7 +89,7 @@ class test_Polaroid:
 
 class test_evcam:
 
-    class MockReceiver(object):
+    class MockReceiver:
         raise_keyboard_interrupt = False
 
         def capture(self, **kwargs):
@@ -106,8 +105,7 @@ class test_evcam:
         self.app.events = self.MockEvents()
         self.app.events.app = self.app
 
-    @mock.restore_logging()
-    def test_evcam(self):
+    def test_evcam(self, restore_logging):
         evcam(Polaroid, timer=timer, app=self.app)
         evcam(Polaroid, timer=timer, loglevel='CRITICAL', app=self.app)
         self.MockReceiver.raise_keyboard_interrupt = True
