@@ -7,7 +7,6 @@ import tempfile
 from unittest.mock import Mock, call, patch
 
 import pytest
-from case import mock
 
 import t.skip
 from celery import _find_option_with_arg, platforms
@@ -18,15 +17,16 @@ from celery.platforms import (ASSUMING_ROOT, ROOT_DISALLOWED,
                               close_open_fds, create_pidlock, detached,
                               fd_by_path, get_fdmax, ignore_errno, initgroups,
                               isatty, maybe_drop_privileges, parse_gid,
-                              parse_uid, set_mp_process_title,
+                              parse_uid, set_mp_process_title, set_pdeathsig,
                               set_process_title, setgid, setgroups, setuid,
                               signals)
 from celery.utils.text import WhateverIO
+from t.unit import conftest
 
 try:
     import resource
 except ImportError:  # pragma: no cover
-    resource = None  # noqa
+    resource = None
 
 
 def test_isatty():
@@ -170,6 +170,18 @@ class test_Signals:
         signals['INT'] = lambda *a: a
 
 
+class test_set_pdeathsig:
+
+    def test_call(self):
+        set_pdeathsig('SIGKILL')
+
+    @t.skip.if_win32
+    def test_call_with_correct_parameter(self):
+        with patch('celery.platforms._set_pdeathsig') as _set_pdeathsig:
+            set_pdeathsig('SIGKILL')
+            _set_pdeathsig.assert_called_once_with(signal.SIGKILL)
+
+
 @t.skip.if_win32
 class test_get_fdmax:
 
@@ -417,7 +429,7 @@ class test_detached:
     @patch('celery.platforms.signals')
     @patch('celery.platforms.maybe_drop_privileges')
     @patch('os.geteuid')
-    @patch(mock.open_fqdn)
+    @patch('builtins.open')
     def test_default(self, open, geteuid, maybe_drop,
                      signals, pidlock):
         geteuid.return_value = 0
@@ -518,7 +530,7 @@ class test_Pidfile:
         p = Pidfile.return_value = Mock()
         p.is_locked.return_value = True
         p.remove_if_stale.return_value = False
-        with mock.stdouts() as (_, err):
+        with conftest.stdouts() as (_, err):
             with pytest.raises(SystemExit):
                 create_pidlock('/var/pid')
             assert 'already exists' in err.getvalue()
@@ -555,14 +567,14 @@ class test_Pidfile:
         assert not p.is_locked()
 
     def test_read_pid(self):
-        with mock.open() as s:
+        with conftest.open() as s:
             s.write('1816\n')
             s.seek(0)
             p = Pidfile('/var/pid')
             assert p.read_pid() == 1816
 
     def test_read_pid_partially_written(self):
-        with mock.open() as s:
+        with conftest.open() as s:
             s.write('1816')
             s.seek(0)
             p = Pidfile('/var/pid')
@@ -572,20 +584,20 @@ class test_Pidfile:
     def test_read_pid_raises_ENOENT(self):
         exc = IOError()
         exc.errno = errno.ENOENT
-        with mock.open(side_effect=exc):
+        with conftest.open(side_effect=exc):
             p = Pidfile('/var/pid')
             assert p.read_pid() is None
 
     def test_read_pid_raises_IOError(self):
         exc = IOError()
         exc.errno = errno.EAGAIN
-        with mock.open(side_effect=exc):
+        with conftest.open(side_effect=exc):
             p = Pidfile('/var/pid')
             with pytest.raises(IOError):
                 p.read_pid()
 
     def test_read_pid_bogus_pidfile(self):
-        with mock.open() as s:
+        with conftest.open() as s:
             s.write('eighteensixteen\n')
             s.seek(0)
             p = Pidfile('/var/pid')
@@ -643,7 +655,7 @@ class test_Pidfile:
 
     @patch('os.kill')
     def test_remove_if_stale_process_dead(self, kill):
-        with mock.stdouts():
+        with conftest.stdouts():
             p = Pidfile('/var/pid')
             p.read_pid = Mock()
             p.read_pid.return_value = 1816
@@ -656,7 +668,7 @@ class test_Pidfile:
             p.remove.assert_called_with()
 
     def test_remove_if_stale_broken_pid(self):
-        with mock.stdouts():
+        with conftest.stdouts():
             p = Pidfile('/var/pid')
             p.read_pid = Mock()
             p.read_pid.side_effect = ValueError()
@@ -667,7 +679,7 @@ class test_Pidfile:
 
     @patch('os.kill')
     def test_remove_if_stale_unprivileged_user(self, kill):
-        with mock.stdouts():
+        with conftest.stdouts():
             p = Pidfile('/var/pid')
             p.read_pid = Mock()
             p.read_pid.return_value = 1817
@@ -692,7 +704,7 @@ class test_Pidfile:
     @patch('os.getpid')
     @patch('os.open')
     @patch('os.fdopen')
-    @patch(mock.open_fqdn)
+    @patch('builtins.open')
     def test_write_pid(self, open_, fdopen, osopen, getpid, fsync):
         getpid.return_value = 1816
         osopen.return_value = 13
@@ -719,7 +731,7 @@ class test_Pidfile:
     @patch('os.getpid')
     @patch('os.open')
     @patch('os.fdopen')
-    @patch(mock.open_fqdn)
+    @patch('builtins.open')
     def test_write_reread_fails(self, open_, fdopen,
                                 osopen, getpid, fsync):
         getpid.return_value = 1816
@@ -825,21 +837,28 @@ class test_setgroups:
             getgroups.assert_called_with()
 
 
+fails_on_win32 = pytest.mark.xfail(
+    sys.platform == "win32",
+    reason="fails on py38+ windows",
+)
+
+
+@fails_on_win32
 @pytest.mark.parametrize('accept_content', [
     {'pickle'},
     {'application/group-python-serialize'},
-    {'pickle', 'application/group-python-serialize'}
+    {'pickle', 'application/group-python-serialize'},
 ])
-def test_check_privileges_suspicious_platform(accept_content):
-    with patch('celery.platforms.os') as os_module:
-        del os_module.getuid
-        del os_module.getgid
-        del os_module.geteuid
-        del os_module.getegid
+@patch('celery.platforms.os')
+def test_check_privileges_suspicious_platform(os_module, accept_content):
+    del os_module.getuid
+    del os_module.getgid
+    del os_module.geteuid
+    del os_module.getegid
 
-        with pytest.raises(SecurityError,
-                           match=r'suspicious platform, contact support'):
-            check_privileges(accept_content)
+    with pytest.raises(SecurityError,
+                       match=r'suspicious platform, contact support'):
+        check_privileges(accept_content)
 
 
 @pytest.mark.parametrize('accept_content', [
@@ -858,157 +877,167 @@ def test_check_privileges(accept_content, recwarn):
     {'application/group-python-serialize'},
     {'pickle', 'application/group-python-serialize'}
 ])
-def test_check_privileges_no_fchown(accept_content, recwarn):
-    with patch('celery.platforms.os') as os_module:
-        del os_module.fchown
-        check_privileges(accept_content)
+@patch('celery.platforms.os')
+def test_check_privileges_no_fchown(os_module, accept_content, recwarn):
+    del os_module.fchown
+    check_privileges(accept_content)
 
     assert len(recwarn) == 0
 
 
+@fails_on_win32
 @pytest.mark.parametrize('accept_content', [
     {'pickle'},
     {'application/group-python-serialize'},
     {'pickle', 'application/group-python-serialize'}
 ])
-def test_check_privileges_without_c_force_root(accept_content):
-    with patch('celery.platforms.os') as os_module:
-        os_module.environ = {}
-        os_module.getuid.return_value = 0
-        os_module.getgid.return_value = 0
-        os_module.geteuid.return_value = 0
-        os_module.getegid.return_value = 0
-
-        expected_message = re.escape(ROOT_DISALLOWED.format(uid=0, euid=0,
-                                                            gid=0, egid=0))
-        with pytest.raises(SecurityError,
-                           match=expected_message):
-            check_privileges(accept_content)
-
-
-@pytest.mark.parametrize('accept_content', [
-    {'pickle'},
-    {'application/group-python-serialize'},
-    {'pickle', 'application/group-python-serialize'}
-])
-def test_check_privileges_with_c_force_root(accept_content):
-    with patch('celery.platforms.os') as os_module:
-        os_module.environ = {'C_FORCE_ROOT': 'true'}
-        os_module.getuid.return_value = 0
-        os_module.getgid.return_value = 0
-        os_module.geteuid.return_value = 0
-        os_module.getegid.return_value = 0
-
-        with pytest.warns(SecurityWarning):
-            check_privileges(accept_content)
-
-
-@pytest.mark.parametrize(('accept_content', 'group_name'), [
-    ({'pickle'}, 'sudo'),
-    ({'application/group-python-serialize'}, 'sudo'),
-    ({'pickle', 'application/group-python-serialize'}, 'sudo'),
-    ({'pickle'}, 'wheel'),
-    ({'application/group-python-serialize'}, 'wheel'),
-    ({'pickle', 'application/group-python-serialize'}, 'wheel'),
-])
-def test_check_privileges_with_c_force_root_and_with_suspicious_group(
-    accept_content, group_name):
-    with patch('celery.platforms.os') as os_module, patch(
-        'celery.platforms.grp') as grp_module:
-        os_module.environ = {'C_FORCE_ROOT': 'true'}
-        os_module.getuid.return_value = 60
-        os_module.getgid.return_value = 60
-        os_module.geteuid.return_value = 60
-        os_module.getegid.return_value = 60
-
-        grp_module.getgrgid.return_value = [group_name]
-        grp_module.getgrgid.return_value = [group_name]
-
-        expected_message = re.escape(ROOT_DISCOURAGED.format(uid=60, euid=60,
-                                                             gid=60, egid=60))
-        with pytest.warns(SecurityWarning, match=expected_message):
-            check_privileges(accept_content)
-
-
-@pytest.mark.parametrize(('accept_content', 'group_name'), [
-    ({'pickle'}, 'sudo'),
-    ({'application/group-python-serialize'}, 'sudo'),
-    ({'pickle', 'application/group-python-serialize'}, 'sudo'),
-    ({'pickle'}, 'wheel'),
-    ({'application/group-python-serialize'}, 'wheel'),
-    ({'pickle', 'application/group-python-serialize'}, 'wheel'),
-])
-def test_check_privileges_without_c_force_root_and_with_suspicious_group(
-    accept_content, group_name):
-    with patch('celery.platforms.os') as os_module, patch(
-        'celery.platforms.grp') as grp_module:
-        os_module.environ = {}
-        os_module.getuid.return_value = 60
-        os_module.getgid.return_value = 60
-        os_module.geteuid.return_value = 60
-        os_module.getegid.return_value = 60
-
-        grp_module.getgrgid.return_value = [group_name]
-        grp_module.getgrgid.return_value = [group_name]
-
-        expected_message = re.escape(ROOT_DISALLOWED.format(uid=60, euid=60,
-                                                            gid=60, egid=60))
-        with pytest.raises(SecurityError,
-                           match=expected_message):
-            check_privileges(accept_content)
-
-
-@pytest.mark.parametrize('accept_content', [
-    {'pickle'},
-    {'application/group-python-serialize'},
-    {'pickle', 'application/group-python-serialize'}
-])
-def test_check_privileges_with_c_force_root_and_no_group_entry(accept_content,
-                                                               recwarn):
-    with patch('celery.platforms.os') as os_module, patch(
-        'celery.platforms.grp') as grp_module:
-        os_module.environ = {'C_FORCE_ROOT': 'true'}
-        os_module.getuid.return_value = 60
-        os_module.getgid.return_value = 60
-        os_module.geteuid.return_value = 60
-        os_module.getegid.return_value = 60
-
-        grp_module.getgrgid.side_effect = KeyError
-
-        expected_message = ROOT_DISCOURAGED.format(uid=60, euid=60,
-                                                   gid=60, egid=60)
+@patch('celery.platforms.os')
+def test_check_privileges_without_c_force_root(os_module, accept_content):
+    os_module.environ = {}
+    os_module.getuid.return_value = 0
+    os_module.getgid.return_value = 0
+    os_module.geteuid.return_value = 0
+    os_module.getegid.return_value = 0
 
+    expected_message = re.escape(ROOT_DISALLOWED.format(uid=0, euid=0,
+                                                        gid=0, egid=0))
+    with pytest.raises(SecurityError,
+                       match=expected_message):
         check_privileges(accept_content)
-        assert len(recwarn) == 2
-
-        assert recwarn[0].message.args[0] == ASSUMING_ROOT
-        assert recwarn[1].message.args[0] == expected_message
 
 
+@fails_on_win32
 @pytest.mark.parametrize('accept_content', [
     {'pickle'},
     {'application/group-python-serialize'},
     {'pickle', 'application/group-python-serialize'}
 ])
-def test_check_privileges_with_c_force_root_and_no_group_entry(accept_content,
-                                                               recwarn):
-    with patch('celery.platforms.os') as os_module, patch(
-        'celery.platforms.grp') as grp_module:
-        os_module.environ = {}
-        os_module.getuid.return_value = 60
-        os_module.getgid.return_value = 60
-        os_module.geteuid.return_value = 60
-        os_module.getegid.return_value = 60
+@patch('celery.platforms.os')
+def test_check_privileges_with_c_force_root(os_module, accept_content):
+    os_module.environ = {'C_FORCE_ROOT': 'true'}
+    os_module.getuid.return_value = 0
+    os_module.getgid.return_value = 0
+    os_module.geteuid.return_value = 0
+    os_module.getegid.return_value = 0
 
-        grp_module.getgrgid.side_effect = KeyError
+    with pytest.warns(SecurityWarning):
+        check_privileges(accept_content)
 
-        expected_message = re.escape(ROOT_DISALLOWED.format(uid=60, euid=60,
-                                                            gid=60, egid=60))
-        with pytest.raises(SecurityError,
-                           match=expected_message):
-            check_privileges(accept_content)
 
-        assert recwarn[0].message.args[0] == ASSUMING_ROOT
+@fails_on_win32
+@pytest.mark.parametrize(('accept_content', 'group_name'), [
+    ({'pickle'}, 'sudo'),
+    ({'application/group-python-serialize'}, 'sudo'),
+    ({'pickle', 'application/group-python-serialize'}, 'sudo'),
+    ({'pickle'}, 'wheel'),
+    ({'application/group-python-serialize'}, 'wheel'),
+    ({'pickle', 'application/group-python-serialize'}, 'wheel'),
+])
+@patch('celery.platforms.os')
+@patch('celery.platforms.grp')
+def test_check_privileges_with_c_force_root_and_with_suspicious_group(
+    grp_module, os_module, accept_content, group_name
+):
+    os_module.environ = {'C_FORCE_ROOT': 'true'}
+    os_module.getuid.return_value = 60
+    os_module.getgid.return_value = 60
+    os_module.geteuid.return_value = 60
+    os_module.getegid.return_value = 60
+
+    grp_module.getgrgid.return_value = [group_name]
+    grp_module.getgrgid.return_value = [group_name]
+
+    expected_message = re.escape(ROOT_DISCOURAGED.format(uid=60, euid=60,
+                                                         gid=60, egid=60))
+    with pytest.warns(SecurityWarning, match=expected_message):
+        check_privileges(accept_content)
+
+
+@fails_on_win32
+@pytest.mark.parametrize(('accept_content', 'group_name'), [
+    ({'pickle'}, 'sudo'),
+    ({'application/group-python-serialize'}, 'sudo'),
+    ({'pickle', 'application/group-python-serialize'}, 'sudo'),
+    ({'pickle'}, 'wheel'),
+    ({'application/group-python-serialize'}, 'wheel'),
+    ({'pickle', 'application/group-python-serialize'}, 'wheel'),
+])
+@patch('celery.platforms.os')
+@patch('celery.platforms.grp')
+def test_check_privileges_without_c_force_root_and_with_suspicious_group(
+    grp_module, os_module, accept_content, group_name
+):
+    os_module.environ = {}
+    os_module.getuid.return_value = 60
+    os_module.getgid.return_value = 60
+    os_module.geteuid.return_value = 60
+    os_module.getegid.return_value = 60
+
+    grp_module.getgrgid.return_value = [group_name]
+    grp_module.getgrgid.return_value = [group_name]
+
+    expected_message = re.escape(ROOT_DISALLOWED.format(uid=60, euid=60,
+                                                        gid=60, egid=60))
+    with pytest.raises(SecurityError,
+                       match=expected_message):
+        check_privileges(accept_content)
+
+
+@fails_on_win32
+@pytest.mark.parametrize('accept_content', [
+    {'pickle'},
+    {'application/group-python-serialize'},
+    {'pickle', 'application/group-python-serialize'}
+])
+@patch('celery.platforms.os')
+@patch('celery.platforms.grp')
+def test_check_privileges_with_c_force_root_and_no_group_entry(
+    grp_module, os_module, accept_content, recwarn
+):
+    os_module.environ = {'C_FORCE_ROOT': 'true'}
+    os_module.getuid.return_value = 60
+    os_module.getgid.return_value = 60
+    os_module.geteuid.return_value = 60
+    os_module.getegid.return_value = 60
+
+    grp_module.getgrgid.side_effect = KeyError
+
+    expected_message = ROOT_DISCOURAGED.format(uid=60, euid=60,
+                                               gid=60, egid=60)
+
+    check_privileges(accept_content)
+    assert len(recwarn) == 2
+
+    assert recwarn[0].message.args[0] == ASSUMING_ROOT
+    assert recwarn[1].message.args[0] == expected_message
+
+
+@fails_on_win32
+@pytest.mark.parametrize('accept_content', [
+    {'pickle'},
+    {'application/group-python-serialize'},
+    {'pickle', 'application/group-python-serialize'}
+])
+@patch('celery.platforms.os')
+@patch('celery.platforms.grp')
+def test_check_privileges_without_c_force_root_and_no_group_entry(
+    grp_module, os_module, accept_content, recwarn
+):
+    os_module.environ = {}
+    os_module.getuid.return_value = 60
+    os_module.getgid.return_value = 60
+    os_module.geteuid.return_value = 60
+    os_module.getegid.return_value = 60
+
+    grp_module.getgrgid.side_effect = KeyError
+
+    expected_message = re.escape(ROOT_DISALLOWED.format(uid=60, euid=60,
+                                                        gid=60, egid=60))
+    with pytest.raises(SecurityError,
+                       match=expected_message):
+        check_privileges(accept_content)
+
+    assert recwarn[0].message.args[0] == ASSUMING_ROOT
 
 
 def test_skip_checking_privileges_when_grp_is_unavailable(recwarn):
