@@ -1,15 +1,11 @@
-# -*- coding: utf-8 -*-
-from __future__ import absolute_import, unicode_literals
-
 import pytest
-from case import skip
 
-from celery.five import text_t
+import t.skip
 from celery.utils import term
 from celery.utils.term import colored, fg
 
 
-@skip.if_win32()
+@t.skip.if_win32
 class test_colored:
 
     @pytest.fixture(autouse=True)
@@ -38,7 +34,7 @@ class test_colored:
         assert str(getattr(colored(), name)('f'))
 
     def test_unicode(self):
-        assert text_t(colored().green('∂bar'))
+        assert str(colored().green('∂bar'))
         assert colored().red('éefoo') + colored().green('∂bar')
         assert colored().red('foo').no_color() == 'foo'
 
