@@ -2,7 +2,7 @@ import sys
 from time import monotonic
 from unittest.mock import Mock, patch
 
-from case import mock
+import pytest
 
 from celery.concurrency.base import BasePool
 from celery.utils.objects import Bunch
@@ -90,16 +90,18 @@ class test_Autoscaler:
 
         worker = Mock(name='worker')
         x = Scaler(self.pool, 10, 3, worker=worker)
-        x._is_stopped.set()
-        x.stop()
+        # Don't allow thread joining or event waiting to block the test
+        with patch("threading.Thread.join"), patch("threading.Event.wait"):
+            x.stop()
         assert x.joined
         x.joined = False
         x.alive = False
-        x.stop()
+        with patch("threading.Thread.join"), patch("threading.Event.wait"):
+            x.stop()
         assert not x.joined
 
-    @mock.sleepdeprived(module=autoscale)
-    def test_body(self):
+    @pytest.mark.sleepdeprived_patched_module(autoscale)
+    def test_body(self, sleepdeprived):
         worker = Mock(name='worker')
         x = autoscale.Autoscaler(self.pool, 10, 3, worker=worker)
         x.body()
@@ -123,13 +125,13 @@ class test_Autoscaler:
 
             def body(self):
                 self.scale_called = True
-                self._is_shutdown.set()
+                getattr(self, "_bgThread__is_shutdown").set()
 
         worker = Mock(name='worker')
         x = Scaler(self.pool, 10, 3, worker=worker)
         x.run()
-        assert x._is_shutdown.isSet()
-        assert x._is_stopped.isSet()
+        assert getattr(x, "_bgThread__is_shutdown").is_set()
+        assert getattr(x, "_bgThread__is_stopped").is_set()
         assert x.scale_called
 
     def test_shrink_raises_exception(self):
@@ -200,7 +202,7 @@ class test_Autoscaler:
         class _Autoscaler(autoscale.Autoscaler):
 
             def body(self):
-                self._is_shutdown.set()
+                getattr(self, "_bgThread__is_shutdown").set()
                 raise OSError('foo')
         worker = Mock(name='worker')
         x = _Autoscaler(self.pool, 10, 3, worker=worker)
@@ -214,8 +216,8 @@ class test_Autoscaler:
         _exit.assert_called_with(1)
         stderr.write.assert_called()
 
-    @mock.sleepdeprived(module=autoscale)
-    def test_no_negative_scale(self):
+    @pytest.mark.sleepdeprived_patched_module(autoscale)
+    def test_no_negative_scale(self, sleepdeprived):
         total_num_processes = []
         worker = Mock(name='worker')
         x = autoscale.Autoscaler(self.pool, 10, 3, worker=worker)
